from django.db import models
from edc_identifier.model_mixins import UniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel
from edc_sites.model_mixins import SiteModelMixin


class Vl(UniqueSubjectIdentifierFieldMixin, SiteModelMixin, BaseUuidModel):
    """A data management table with details of each HIV participant's
    viral load.

    """

    baseline_date = models.DateField(null=True)

    baseline_vl_date = models.DateField(null=True)
    endline_vl_date = models.DateField(null=True)

    min_vl_date = models.DateField(null=True)
    max_vl_date = models.DateField(null=True)

    baseline_vl = models.IntegerField(null=True)
    endline_vl = models.IntegerField(null=True)



from django.contrib import admin
from django.contrib.admin import ModelAdmin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ...admin_site import intecomm_subject_admin
from ...models import CurrentConditions


@admin.register(CurrentConditions, site=intecomm_subject_admin)
class CurrentConditionsAdmin(ModelAdminSubjectDashboardMixin, ModelAdmin):
    list_display = (
        "subject_identifier",
        "dashboard",
        "baseline_date",
        "hiv",
        "htn",
        "dm",
        "created",
        "comment",
    )
    list_filter = ("hiv", "htn", "dm")
    readonly_fields = ("subject_identifier", "hiv", "htn", "dm")



from django.db import models
from django_pandas.managers import DataFrameManager
from edc_identifier.model_mixins import UniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel
from edc_sites.model_mixins import SiteModelMixin


class CurrentConditions(UniqueSubjectIdentifierFieldMixin, SiteModelMixin, BaseUuidModel):
    """A data management table with details of each participant's
    diagnoses.

    Run `python manage.py update_conditions` to populate.
    """

    baseline_date = models.DateField(null=True)

    hiv = models.BooleanField(default=False)
    htn = models.BooleanField(default=False)
    dm = models.BooleanField(default=False)

    hiv_dx_date = models.DateField(null=True)
    htn_dx_date = models.DateField(null=True)
    dm_dx_date = models.DateField(null=True)

    hiv_dx_days = models.IntegerField(
        default=None, null=True, help_text="Number of days relative to baseline"
    )
    dm_dx_days = models.IntegerField(
        default=None, null=True, help_text="Number of days relative to baseline"
    )
    htn_dx_days = models.IntegerField(
        default=None, null=True, help_text="Number of days relative to baseline"
    )

    comment = models.TextField(null=True)

    objects = DataFrameManager()

    def update_site_on_save(self, *args, **kwargs):
        pass

    class Meta(UniqueSubjectIdentifierFieldMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Current Conditions"
        verbose_name_plural = "Current Conditions"



# Generated by Django 4.2.11 on 2024-06-17 12:01

from django.db import migrations
from django.db.migrations import RunPython
from edc_constants.constants import FEMALE, MALE
from edc_utils import get_utcnow


def update_gender(apps, schema_editor):
    """Update gender from 'F' to 'M' for participant added to patient
    log (and subsequently screened and consented) with the wrong gender.

    See also:
        Redmine ticket: #810
        Protocol incident: VWFD-NPT2
    """
    for model in ["intecomm_consent.subjectconsent", "edc_registration.registeredsubject"]:
        model_cls = apps.get_model(model)
        obj = model_cls.objects.get(
            subject_identifier="107-105-0080-1",
            screening_identifier="SYKEZZEV",
            site_id=105,
            gender=FEMALE,
        )
        print(
            f"\n  * Updating {model_cls.__name__} gender for {obj.subject_identifier=} "
            f"({obj.screening_identifier=}) from '{FEMALE}' to '{MALE}' ..."
        )

        obj.modified = get_utcnow()
        obj.user_modified = __name__ if len(__name__) <= 50 else f"{__name__[:46]} ..."
        obj.gender = MALE
        obj.save_base(
            update_fields=[
                "gender",
                "modified",
                "user_modified",
            ]
        )

        print("  * Updated.")
    print("Done.")


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_consent", "0025_alter_historicalsubjectconsent_guardian_name_and_more"),
        ("intecomm_screening", "0072_data_fix_810_gender_update"),
    ]

    operations = [RunPython(update_gender)]



from datetime import date

from django.db.models import Count, Min
from edc_dx.diagnoses import (
    ClinicalReviewBaselineRequired,
    Diagnoses,
    DiagnosesError,
    InitialReviewRequired,
)
from edc_dx_review.model_mixins.factory.calculate_date import update_calculated_date
from edc_model.utils import get_report_datetime_field_name
from edc_utils import get_utcnow
from tqdm import tqdm

from .models import (
    CurrentConditions,
    DmInitialReview,
    HivInitialReview,
    HtnInitialReview,
    SubjectVisit,
)


def recalculate_dx_calculated_date(
    models: dict | None = None, subject_identifier: str | None = None
):
    errors = []
    models = models or {
        "hiv": HivInitialReview,
        "htn": HtnInitialReview,
        "dm": DmInitialReview,
    }
    for fld_prefix, model_cls in models.items():
        opts = {}
        disable_tqdm = False
        if subject_identifier:
            opts = dict(subject_visit__subject_identifier=subject_identifier)
            disable_tqdm = True
        qs = model_cls.objects.filter(**opts)
        total = qs.count()
        for obj in tqdm(qs, total=total, disable=disable_tqdm):
            subject_identifier = obj.subject_visit.subject_identifier
            update_calculated_date(
                obj,
                fld_prefix="dx",
                reference_field=get_report_datetime_field_name(),
            )
            obj.modified = get_utcnow()
            obj.user_modified = "erikvw"
            try:
                obj.save_base(
                    update_fields=[
                        "dx_calculated_date",
                        "dx_date_is_estimated",
                        "modified",
                        "user_modified",
                    ]
                )
            except DiagnosesError as e:
                errors.append(f"{subject_identifier}. {e}")
    for err in errors:
        print(err)


def update_current_conditions(delete_all: bool | None = None):
    """Calculate the diagnoses of each participant who has completed
    a baseline visit.

    To be considered having completed the baseline visit, the
    ClinicalReviewBaseline and the corresponding IntialReview forms
    must be complete.

    Protocol requires participants be diagnosed at least 6m ago and
    stable in care for at least 6 months. When querying the InitialReview
    forms, the diagnosis date must be at least 180 days prior to the
    baseline visit date.
    """
    errors = []
    if delete_all:
        CurrentConditions.objects.all().delete()
    qs = (
        SubjectVisit.objects.values("subject_identifier", "site_id")
        .annotate(
            Count("subject_identifier", distinct=True),
            Count("site_id", distinct=True),
        )
        .order_by("subject_identifier")
        .annotate(Min("report_datetime"))
    )
    total = qs.count()
    for obj in tqdm(qs, total=total):
        current_condition, _ = CurrentConditions.objects.get_or_create(
            subject_identifier=obj.get("subject_identifier"), site_id=obj.get("site_id")
        )
        baseline_date = obj.get("report_datetime__min").date()
        errors.extend(update_current_condition(current_condition, baseline_date))
    for err in errors:
        print(err)


def update_current_condition(current_condition, baseline_date: date) -> list:
    """Updates for a single condition.

    Called by `update_current_conditions`.
    """
    errors = []
    current_condition.baseline_date = baseline_date
    try:
        diagnoses = Diagnoses(
            subject_identifier=current_condition.subject_identifier,
            report_datetime=get_utcnow(),
            lte=True,
        )
    except ClinicalReviewBaselineRequired as e:
        errors.append(f"{current_condition.subject_identifier}: {e}")
    else:
        try:
            initial_reviews = diagnoses.initial_reviews
        except InitialReviewRequired:
            errors.append(
                f"{current_condition.subject_identifier}: initial_reviews not found!"
            )
        else:
            for prefix in diagnoses.diagnosis_labels:
                if initial_review := initial_reviews.get(prefix):
                    recalculate_dx_calculated_date(
                        models={prefix: initial_review.__class__},
                        subject_identifier=current_condition.subject_identifier,
                    )
                    initial_review.refresh_from_db()
                    if dx_date := (
                        getattr(initial_review, "dx_date")
                        or getattr(initial_review, "dx_calculated_date")
                    ):
                        setattr(current_condition, prefix, True)
                        setattr(current_condition, f"{prefix}_dx_date", dx_date)
                        dx_days = (dx_date - current_condition.baseline_date).days
                        setattr(current_condition, f"{prefix}_dx_days", dx_days)
                    else:
                        errors.append(
                            f"{current_condition.subject_identifier}: "
                            f"`dx_calculated_date` is None. Got {prefix}: "
                            f"{getattr(initial_review, 'dx_date')}"
                            f"{getattr(initial_review, 'dx_ago')}"
                        )
                        current_condition.comment = ",".join(errors)
                        setattr(current_condition, prefix, False)
                        setattr(current_condition, f"{prefix}_dx_date", None)
                        setattr(current_condition, f"{prefix}_dx_days", None)

    current_condition.save()
    return errors
